var Class               = require('pseudoclass'),
    UIKevWebEditor      = require('../ui/UIKevWebEditor'),
    kevoree             = require('kevoree-library').org.kevoree,
    Kotlin              = require('kevoree-kotlin'),
    ModelEventProcessor = require('./ModelEventProcessor'),
    KevsParser          = require('./kevscript/KevsParser'),
    KevsHint            = require('./kevscript/KevsHint');

/**
 * Created by leiko on 23/01/14.
 */
var KevWebEditor = Class({
    toString: 'KevWebEditor',

    construct: function () {
        var factory = new kevoree.factory.DefaultKevoreeFactory();
        this.model = factory.createContainerRoot();
        factory.root(this.model);
        this.compare = factory.createModelCompare();
        this.tracker = new kevoree.modeling.api.util.ModelTracker(this.compare);
        this.ui = new UIKevWebEditor(this);
        this.draggedElement = null;

        this.tracker.track(this.model);
        this.eventProcessor = new ModelEventProcessor(this, this.ui);
        this.modelVisitor = new kevoree.modeling.api.util.ModelVisitor();
        this.modelVisitor.visit = function (elem) {
            // if elem has an ui in the editor, ask its eventProcessor to do postProcess
            // in order to apply the changes made by the merge
            if (elem.ui) {
                elem.ui.eventProcessor.doPostProcess();
            }

            // visit fragDic element to register a listener on each (or to update name accordingly on changes)
            if (Kotlin.isType(elem, kevoree.FragmentDictionary)) {
                var node = this.model.findNodesByID(elem.name);
                if (node) {
                    node.addModelElementListener({
                        elementChanged: function (event) {
                            if (event.etype === kevoree.modeling.api.util.ActionType.object.SET && event.elementAttributeName === 'name') {
                                elem.name = event.value;
                            }
                        }
                    });
                }
            }
        }.bind(this);
        this.processor = this.eventProcessor.processor();
        new KevsParser(this);
        new KevsHint(this);
    },

    setModel: function (model) {
        var start = new Date().getTime();
        this.ui.cancelUIUpdates();
        this.ui.clean();
        this.tracker.untrack();

        this.model = model;
        var factory = new kevoree.factory.DefaultKevoreeFactory();
        factory.root(this.model);
        this.model.visit(this.modelVisitor, true, true, false);
        this.tracker.track(this.model);
        this.eventProcessor.setModel(this.model);
        this.model.addModelElementListener(this.processor);
        this.ui.enableUIUpdates();
        this.ui.update();
        this.ui.hideLoadingLayer();
        var end = new Date().getTime();
        console.log('editor.setModel() took '+(end - start)+'ms');
    },
    
    suspendModelListener: function () {
        this.model.removeModelElementListener(this.processor);
    },
    
    enableModelListener: function () {
        this.model.addModelElementListener(this.processor);
    },

    setDraggedElement: function (elem) {
        this.draggedElement = elem;
    },

    getDraggedElement: function () {
        return this.draggedElement;
    },

    setDroppableContainerNode: function (node) {
        this.availableContainerNode = node;
    },

    getDroppableContainerNode: function () {
        return this.availableContainerNode;
    },
    
    clearAll: function () {
        var factory = new kevoree.factory.DefaultKevoreeFactory();
        var model = factory.createContainerRoot();
        factory.root(model);
        this.setModel(model);
    },
    
    clearInstances: function () {
        this.model.removeAllNodes();
        this.model.removeAllGroups();
        this.model.removeAllHubs();
        this.model.removeAllMBindings();
        this.ui.clean();
    },

    clearUnusedTDefs: function () {
        // TODO update according to Kevoree MM v5
//        this.ui.cancelUIUpdates();
//        this.ui.showLoadingLayer();
//        var usedTdefs = [];
//
//        function addUsedTDef(elements) {
//            while (elements.hasNext()) {
//                var elem = elements.next();
//                if (elem.components && elem.components.size() > 0) {
//                    // elem is a node: check components typeDefinition
//                    addUsedTDef(elem.components.iterator());
//                }
//
//                if (elem.subNodes && elem.subNodes.size() > 0) {
//                    // elem is a node: check subNodes typeDefinition
//                    addUsedTDef(elem.subNodes.iterator());
//
//                }
//
//                if (usedTdefs.indexOf(elem.typeDefinition) === -1) {
//                    usedTdefs.push(elem.typeDefinition);
//                }
//            }
//        }
//        addUsedTDef(this.model.nodes.iterator());
//        addUsedTDef(this.model.groups.iterator());
//        addUsedTDef(this.model.hubs.iterator());
//
//        var tDefsToDelete = [], tdefs, tdef;
//        tdefs = this.model.select('**/typeDefinitions[name=*]').iterator();
//        while (tdefs.hasNext()) {
//            tdef = tdefs.next();
//            if (usedTdefs.indexOf(tdef) === -1) {
//                tDefsToDelete.push(tdef);
//            }
//        }
//
//        for (var i=0; i < tDefsToDelete.length; i++) {
//            // remove TypeDefinition
//            this.model.removeTypeDefinitions(tDefsToDelete[i]);
//            // remove Library
//            var packages = this.model.select('**/packages[*]').iterator();
//            while (packages.hasNext()) {
//                var lib = packages.next();
//                lib.removeSubTypes(tDefsToDelete[i]);
//                if (lib.subTypes.size() === 0) {
//                    this.model.removeLibraries(lib);
//                }
//            }
//            // remove DeployUnit
//            var removeDU = true;
//            tdefs = this.model.select('**/typeDefinitions[name=*]').iterator();
//            while (tdefs.hasNext()) {
//                tdef = tdefs.next();
//                var dus = tdef.deployUnits.iterator();
//                while (dus.hasNext()) {
//                    var du = dus.next();
//
//                }
//                if (tdef.deployUnit.name === tDefsToDelete[i].deployUnit.name) {
//                    removeDU = false;
//                }
//            }
//            if (removeDU) {
//                this.model.removeDeployUnits(tDefsToDelete[i].deployUnit);
//            }
//        }
//
//        this.ui.enableUIUpdates();
//        this.ui.update();
//        this.ui.hideLoadingLayer();
    },

    undo: function () {
        this.tracker.undo();
    },

    redo: function () {
        this.tracker.redo();
    },

    mergeModel: function (model) {
        throw new Error('Deprecated for performance reasons');
//        var start = new Date().getTime();
//
//        this.ui.cancelUIUpdates();
//        this.delayEventProcessor = true;
//        this.compare.merge(this.model, model).applyOn(this.model);
//        var factory = new kevoree.factory.DefaultKevoreeFactory();
//        factory.root(this.model);
//        this.delayEventProcessor = false;
//
//        // do some process over all model elements
//        this.model.visit(this.modelVisitor, true, true, false);
//
//        // call postProcess on ModelEventProcessor when merge is done
//        this.eventProcessor.doPostProcess();
//
//        // re-attach model listener
//        this.suspendModelListener();
//        this.enableModelListener();
//
//        // update ui
//        this.ui.enableUIUpdates();
//        this.ui.update();
//        this.ui.hideLoadingLayer();
//
//        var end = new Date().getTime();
//        console.log('editor.mergeModel() took '+(end - start)+'ms');
    },

    getModel: function () {
        return this.model;
    },

    getUI: function () {
        return this.ui;
    },
    
    delayingEventProcessor: function () {
        return this.delayEventProcessor;
    }
});

module.exports = KevWebEditor;