var HostedEntity    = require('./HostedEntity'),
    AbstractEntity  = require('./AbstractEntity'),
    UIComponent     = require('../ui/UIComponent'),
    DeleteNode      = require('../command/model/DeleteNode'),
    OpenNodeProps   = require('../command/ui/OpenNodeProps'),
    ModelHelper     = require('../util/ModelHelper'),
    ModelChecker    = require('../util/ModelChecker'),
    //Alert           = require('../util/Alert'),
    UIWire          = require('./UIWire');

// CONSTANTS
var PADDING = 12,
    SELECTED_COLOR = 'orange',
    NORMAL_COLOR = 'white';

/**
 * Created by leiko on 27/01/14.
 */
var UINode = HostedEntity.extend({
    toString: 'UINode',

    construct: function (instance, editor) {
        this.wires = [];
        this.text = new Kinetic.Text({
            text:       instance.name + ' : ' + instance.typeDefinition.name,
            fontSize:   13,
            fontFamily: 'Helvetica',
            fill:       (this.instance.started) ? '#fff' : 'black',
            padding:    PADDING,
            align:      'center'
        });
        this.defaultWidth = this.text.getWidth();

        this.bgRect = new Kinetic.Rect({
            name:    'droppable-instance',
            width:   this.text.getWidth(),
            height:  this.text.getHeight(),
            fill:    'white',
            cornerRadius:  5,
            opacity: 0.1
        });

        this.bgRect.sceneFunc(function (context) {
            this.refreshShapeAttributes();
            this.bgRect._sceneFunc(context);
        }.bind(this));

        this.border = new Kinetic.Rect({
            width:         this.bgRect.getWidth(),
            height:        this.bgRect.getHeight(),
            stroke:        NORMAL_COLOR,
            strokeWidth:   2,
            cornerRadius:  10,
            shadowColor:   'black',
            shadowBlur:    10,
            shadowOffset:  {x: 3, y: 3},
            shadowOpacity: 0.2
        });

        this.shape.add(this.bgRect);
        this.shape.add(this.border);
        this.shape.add(this.text);

        var components = instance.components.iterator();
        while (components.hasNext()) {
            var comp = components.next();
            if (!comp.ui) {
                var uiComp = new UIComponent(comp, editor, instance);
                this.shape.add(uiComp.getShape());
            } else {
                comp.ui.update();
            }
        }

        var hostedNodes = instance.hosts.iterator();
        while (hostedNodes.hasNext()) {
            var node = hostedNodes.next();
            if (!node.ui) {
                var uiNode = new UINode(node, editor, instance);
                this.shape.add(uiNode.getShape());
            } else {
                node.ui.update();
            }
        }

        //var compAlert = new Alert();
        this.shape.on('mouseover touchmove', function (e) {
            e.cancelBubble = true;
            var draggedEntity = editor.getDraggedElement();
            if (draggedEntity && this.shape.getLayer()) {
                if (draggedEntity.type === 'wire') {
                    var srcInstance = draggedEntity.ui.getSrcInstance();
                    if (ModelHelper.findInstanceType(srcInstance) === 'group') {
                        var grps = instance.groups.iterator();
                        while (grps.hasNext()) {
                            if (grps.next().name === srcInstance.name) {
                                this.border.setStroke('#F00');
                                this.shape.getLayer().batchDraw();
                                return;
                            }
                        }
                        this.border.setStroke('#0F0');
                        this.shape.getLayer().batchDraw();
                    }

                } else if (draggedEntity.type === 'component' || draggedEntity.type === 'node') {
                    var tdef = ModelHelper.findTypeDefinitionsByFQN(draggedEntity.fqn, editor.getModel()).get(0);
                    var value = ModelChecker.checkPlatform(tdef, instance.typeDefinition);
                    if (value) {
                        this.border.setStroke('#0F0');
                        this.shape.getLayer().batchDraw();
                    } else {
                        //compAlert.setType(Alert.WARNING);
                        //compAlert.setText('Unable to add component', 'Instance '+instance.name+' does not support this component');
                        //compAlert.show(5000);
                        this.border.setStroke('#F00');
                        this.shape.getLayer().batchDraw();
                    }
                } else {
                    this.border.setStroke('#F00');
                    this.shape.getLayer().batchDraw();
                }
            }
        }.bind(this));

        this.shape.on('mouseout mousenotover touchend', function (e) {
            if (this.isSelected) {
                this.border.setStroke(SELECTED_COLOR);
                this.shape.getLayer().batchDraw();
            } else if (this.shape.getLayer()) {
                //compAlert.hideNow();
                this.border.setStroke(NORMAL_COLOR);
                this.shape.getLayer().batchDraw();
            }
        }.bind(this));

        this.shape.on('mouseup touchend', function (e) {
            this.shape.getStage().setDraggable(true);
            if (this.isSelected) {
                this.border.setStroke(SELECTED_COLOR);
                this.shape.getLayer().batchDraw();
            } else if (this.shape.getLayer()) {
                this.border.setStroke(NORMAL_COLOR);
                this.shape.getLayer().batchDraw();
            }

            var draggedElem = editor.getDraggedElement();
            if (draggedElem) {
                if (draggedElem.type === 'wire') {
                    console.log('MOUSEUP', 'WIRE');
                    var srcInstance = draggedElem.ui.getSrcInstance();
                    if (ModelHelper.findInstanceType(srcInstance) === 'group') {
                        var uiWire = draggedElem.ui;
                        srcInstance.addSubNodes(instance);
                        uiWire.getShape().remove();
                        editor.getUI().getWiresLayer().batchDraw();
                        editor.setDraggedElement(null);
                    }

                } else if (draggedElem.typeDef && (draggedElem.type === 'component' || draggedElem.type === 'node')) {
                    console.log('MOUSEUP', 'COMP|NODE');
                    var tdef = ModelHelper.findTypeDefinitionsByFQN(draggedElem.fqn, editor.getModel()).get(0);
                    var value = ModelChecker.checkPlatform(tdef, instance.typeDefinition);
                    if (value) {
                        // this code is triggered by a drag'n'drop of a Node or Component from TypeDefList to this node ui
                        // if editor.getDroppableContainerNode() is already set, it means that another node has
                        // already answered positively to that event, so this one does not have to process it
                        if (!editor.getDroppableContainerNode()) {
                            // but if editor.getDroppableContainerNode() == null, then this node will be the container
                            editor.setDroppableContainerNode(instance);
                        }
                    } else {
                        // component is not supposed to run on this platform node
                        console.log('PLATFORM NOT SUPPORTED BY THIS COMP');
                    }
                } else {
                    console.log('MOUSEUP', 'ELSE');
                    if (draggedElem.instance.path() !== instance.path()) {
                        draggedElem.shape.remove();
                        if (draggedElem.type === 'component') {
                            instance.addComponents(draggedElem.instance);

                        } else if (draggedElem.type === 'node') {
                            instance.addHosts(draggedElem.instance);
                            e.cancelBubble = true;
                            editor.setDraggedElement(null);
                        }
                    }
                }
            }
        }.bind(this));

        this.openPropsCmd = new OpenNodeProps(editor, this);

        this.shape.on('dragstart', function (e) {
            e.cancelBubble = true;
            editor.setDraggedElement({
                type: 'node',
                instance: instance,
                shape: this.shape,
                typeDef: false,
                fqn: ModelHelper.getFQN(instance)
            });

            if (instance.host) {
                // starting to drag an hosted node
                var position = this.shape.getAbsolutePosition();
                var layer = this.shape.getLayer();
                instance.host.removeHosts(instance);
                layer.add(this.shape);
                this.shape.setPosition(position);
            }

            this.shape.setZIndex(0);
        }.bind(this));

        this.shape.on('dragend', function (e) {
            // if we end-up here, it means that this node has no host (otherwise, its host would have consume the event)
            e.cancelBubble = true;
            editor.setDraggedElement(null);
        }.bind(this));

        this.shape.on('dragmove', function () {
            editor.getUI().getWiresLayer().batchDraw();

            var pointerPos = editor.getUI().getStage().getPointerPosition();
            var nodeShapes = editor.getUI().getStage().find('.droppable-instance');
            var topLevelShape = null;
            for (var i=0; i < nodeShapes.length; i++) {
                if (nodeShapes[i].getParent().id() !== instance.path()) {
                    if (nodeShapes[i].intersects(pointerPos)) {
                        if (!topLevelShape || (topLevelShape.getZIndex() < nodeShapes[i].getParent().getZIndex())) {
                            if (topLevelShape) {
                                topLevelShape.fire('mousenotover');
                            }
                            topLevelShape = nodeShapes[i].getParent();
                        }
                    } else {
                        nodeShapes[i].getParent().fire('mousenotover');
                    }
                }
            }
            if (topLevelShape) {
                topLevelShape.fire('mouseover');
            }
        }.bind(this));

        console.log('UINode', this.shape);

        this.deleteCmd = new DeleteNode(editor);
    },

    /**
     * This method is called each time KineticJS judge necessary to redraw the shape
     */
    refreshShapeAttributes: function () {
        var components  = this.instance.components.iterator(),
            hosts       = this.instance.hosts.iterator(), // subnodes
            yOffset     = this.text.getHeight(),
            maxWidth    = this.bgRect.getWidth() - PADDING*2;

        if (this.instance.components.size() === 0 && this.instance.hosts.size() === 0 && this.instance.host === null) {
            this.setWidth(this.defaultWidth);
        }

        var hostedUIs = [];
        while (components.hasNext()) {
            hostedUIs.push(components.next().ui);
        }
        while (hosts.hasNext()) {
            hostedUIs.push(hosts.next().ui);
        }

        for (var i=0; i < hostedUIs.length; i++) {
            if (hostedUIs[i] && hostedUIs[i].getShape().isVisible()) {
                if (hostedUIs[i].getWidth() >= maxWidth) {
                    maxWidth = hostedUIs[i].getWidth() + PADDING*2;
                    this.setWidth(maxWidth);
                } else {
                    hostedUIs[i].setWidth(this.bgRect.getWidth() - PADDING*2);
                }

                // compute hosted entity group shape x and y position in node group shape
                hostedUIs[i].setX(this.getWidth()/2 - (hostedUIs[i].getWidth() / 2));
                hostedUIs[i].setY(yOffset);
                yOffset += hostedUIs[i].getHeight() + PADDING;
                // end compute position
            }
        }

        // set new node group shape height & width
        this.bgRect.setHeight(yOffset);
        this.border.setHeight(yOffset);
        this.bgRect.setWidth(this.text.getWidth());
        this.border.setWidth(this.text.getWidth());
    },

    onRemoveHostedInstance: function (instance) {
        instance.ui.getShape().remove();
    },

    onAdd: function (instance) {
        if (instance) {
            var type = ModelHelper.findInstanceType(instance);
            var uiEntity = instance.ui;
            switch (type) {
                case 'component':
                    if (!uiEntity) uiEntity = new UIComponent(instance, this.editor);
                    break;

                case 'node':
                    if (!uiEntity) uiEntity = new UINode(instance, this.editor);
                    break;
            }
            this.shape.add(uiEntity.getShape());
            this.editor.getUI().update();
        }
    },

    getPlugPosition: function (grpPlugPos) {
        var pos = this.shape.getAbsolutePosition();

        var plugPos = {
            x: pos.x + PADDING/2,
            y: pos.y + PADDING/2
        };

        if (grpPlugPos && ((grpPlugPos.x - (this.getWidth()/2)) > pos.x)) {
            // if origin point is on the right, then give the upper right corner for node wire's plug
            plugPos.x = pos.x + this.getWidth() - 5;
        }

        // default is the upper left corner for node wire's plug
        return plugPos;
    },

    update: function () {
        if (this.shape.getLayer()) {
            this.shape.id(this.instance.path());
            if (this.instance.name.length > 15) {
                this.text.text(this.instance.name + '\n' + this.instance.typeDefinition.name);
            } else {
                this.text.text(this.instance.name + ' : ' + this.instance.typeDefinition.name);
            }
            this.text.fill((this.instance.started) ? '#fff' : 'black');
            this.shape.getLayer().batchDraw();
        }
    },

    onSet: function (attrName, sourcePath, previousVal, value) {
        console.log('SET UINode', this.instance.name, attrName, sourcePath, previousVal, value);
        if (typeof(value) !== 'undefined' && value != null) {
            if (attrName === 'name') {
                function updateFragDic(instances) {
                    while (instances.hasNext()) {
                        var inst = instances.next();
                        var dic = inst.findFragmentDictionaryByID(previousVal);
                        if (dic) dic.name = value;
                    }
                }
                updateFragDic(this.editor.getModel().groups.iterator());
                updateFragDic(this.editor.getModel().hubs.iterator());
            }
            this.update();
        }
    },

    onGroupRemoved: function (group) {
        for (var i=0; i < group.ui.wires.length; i++) {
            if (group.ui.wires[i].getTargetInstance().name === this.instance.name) {
                this.removeWire(group.ui.wires[i]);
            }
        }
    },

    onDelete: function () {
        this._super();
        var comps = this.instance.components.iterator();
        while (comps.hasNext()) comps.next().ui.onDelete();
        var subNodes = this.instance.hosts.iterator();
        while (subNodes.hasNext()) subNodes.next().ui.onDelete();
    },

    contains: function (point) {
        return this.bgRect.intersects(point);
    },

    show: function (_super) {
        this._super();
        var comps = this.instance.components.iterator();
        while (comps.hasNext()) {
            comps.next().ui.show();
        }
        var subNodes = this.instance.hosts.iterator();
        while (subNodes.hasNext()) {
            subNodes.next().ui.show();
        }
    },

    hide: function (_super) {
        this._super();
        var comps = this.instance.components.iterator();
        while (comps.hasNext()) {
            comps.next().ui.hide();
        }
        var subNodes = this.instance.hosts.iterator();
        while (subNodes.hasNext()) {
            subNodes.next().ui.hide();
        }
    },

    setSelected: function (boolean) {
        if (boolean) {
            this.border.setStroke(SELECTED_COLOR);
            this.shape.getLayer().batchDraw();
        } else {
            this.border.setStroke(NORMAL_COLOR);
            this.shape.getLayer().batchDraw();
        }
    }
});

module.exports = UINode;
